@@ -5,36 +5,21 @@ from .artist import Artist
 from .artists import Artists
 from .album_list import AlbumList
 from .album import Album
+from .song import Song
 from .random_songs import RandomSongs
 from .search_result import SearchResult
 from .artist_cover import ArtistCover
 from .genres import Genres
+from .response import Response
+from .list_type import ListType
+
+from .configuration import Configuration
 
 class Connector:
     
-    __KEY_BASE_URL = "baseUrl"
-    __KEY_PORT = "port"
-    __KEY_USERNAME = "username"
-    __KEY_PASSWORD = "password"
-    __KEY_API_VERSION = "apiVersion"
-    __KEY_APP_NAME = "appName"
+    def __init__(self, configuration : Configuration):
+        self.__configuration = configuration
 
-    def __init__(
-            self,
-            baseUrl : str,
-            port : int,
-            username : str,
-            password : str,
-            apiVersion : str = libsonic.API_VERSION,
-            appName : str = "navibridge"):
-        self.__config__ = {}
-        self.__config__[Connector.__KEY_BASE_URL] = baseUrl
-        self.__config__[Connector.__KEY_PORT] = port
-        self.__config__[Connector.__KEY_USERNAME] = username
-        self.__config__[Connector.__KEY_PASSWORD] = password
-        self.__config__[Connector.__KEY_API_VERSION] = apiVersion
-        self.__config__[Connector.__KEY_APP_NAME] = appName
-        
     def getIndexes(self, 
             musicFolderId = None, 
             ifModifiedSince = 0):
@@ -42,11 +27,13 @@ class Connector:
             musicFolderId = musicFolderId, 
             ifModifiedSince = ifModifiedSince)
 
-    def getArtists(self) -> Artists:
-        return Artists(self.__connect().getArtists())
+    def getArtists(self) -> Response[Artists]:
+        data = self.__connect().getArtists()
+        return Response(data, Artists(data) if data else None)
 
-    def getGenres(self) -> Genres:
-        return Genres(self.__connect().getGenres())
+    def getGenres(self) -> Response[Genres]:
+        data = self.__connect().getGenres()
+        return Response(data, Genres(data) if data else None)
 
     def getCoverArtForGenre(self,
             genre : str,
@@ -58,7 +45,6 @@ class Connector:
             raise Exception("No songs for genre [{}]"
                 .format(genre))
         songList = Item(songs).getList(["songsByGenre", "song"])
-        #songList : list = songs["songsByGenre"]["song"]
         if not songList: return None
         coverArt : str = ""
         for current in songList:
@@ -69,37 +55,42 @@ class Connector:
             size = 10, 
             genre = None, 
             fromYear = None, toYear = None, 
-            musicFolderId = None) -> RandomSongs:
-        return RandomSongs(self.__connect().getRandomSongs(
+            musicFolderId = None) -> Response[RandomSongs]:
+        data : dict = self.__connect().getRandomSongs(
             size = size, 
             genre = genre, 
             fromYear = fromYear, toYear = toYear, 
-            musicFolderId = musicFolderId))
+            musicFolderId = musicFolderId)
+        return Response(data, RandomSongs(data) if data else None)
 
     def getAlbumList(self, 
-            ltype, 
+            ltype : ListType = None, 
             size = 10, offset = 0, 
             fromYear = None, toYear = None, 
-            genre = None, musicFolderId = None) -> AlbumList:
-        return AlbumList(self.__connect().getAlbumList(
-            ltype = ltype, 
+            genre = None, musicFolderId = None) -> Response[AlbumList]:
+        data : dict = self.__connect().getAlbumList(
+            ltype = ltype.getArgValue() if ltype else None, 
             size = size, offset = offset, 
             fromYear = fromYear, toYear = toYear,
-            genre = genre, musicFolderId = musicFolderId))
+            genre = genre, musicFolderId = musicFolderId)
+        return Response(data, AlbumList(data) if data else None)
 
     def getAlbum(self,
-            albumId : str) -> Album | None:
-        album = self.__connect().getAlbum(albumId)
-        return (Album(album) 
-            if album 
-            else None)
-            
+            albumId : str) -> Response[Album]:
+        data = self.__connect().getAlbum(albumId)
+        return Response(data, Album(data) if data else None)
+
+    def getSong(self,
+            song_id : str) -> Response[Song]:
+        data = self.__connect().getSong(song_id)
+        return Response(data, Song(data) if data else None)
+
     def getNewestAlbumList(self, 
             size = 10, offset = 0, 
             fromYear = None, toYear = None, 
-            genre = None, musicFolderId = None) -> AlbumList:
+            genre = None, musicFolderId = None) -> Response[AlbumList]:
         return self.getAlbumList(
-            ltype = "newest", 
+            ltype = ListType.NEWEST, 
             size = size, offset = offset, 
             fromYear = fromYear, toYear = toYear,
             genre = genre, musicFolderId = musicFolderId)
@@ -107,19 +98,23 @@ class Connector:
     def getRandomAlbumList(self, 
             size = 10, offset = 0, 
             fromYear = None, toYear = None, 
-            genre = None, musicFolderId = None) -> AlbumList:
+            genre = None, musicFolderId = None) -> Response[AlbumList]:
         return self.getAlbumList(
-            ltype = "random", 
+            ltype = ListType.RANDOM, 
             size = size, offset = offset, 
             fromYear = fromYear, toYear = toYear,
             genre = genre, musicFolderId = musicFolderId)
 
-    def getArtist(self, artist_id : str) -> Artist:
-        return Artist(self.__connect().getArtist(artist_id))
+    def getArtist(self, artist_id : str) -> Response[Artist]:
+        data = self.__connect().getArtist(artist_id)
+        return Response(data, Artist(data) if data else None)
 
-    def getArtistCover(self, artist : Artist) -> ArtistCover | None:
-        first_album : str
-        first_album_cover_art : str
+    def getCoverByArtistId(self, artist_id : str) -> ArtistCover | None:
+        artist_response : Response[Artist] = self.getArtist(artist_id)
+        if not artist_response and not artist_response.getObj(): return None
+        return self.getCoverByArtist(artist_response.getObj())
+
+    def getCoverByArtist(self, artist : Artist) -> ArtistCover | None:
         album_list : list[Album] = artist.getAlbumList()
         current : Album
         for selected_album in album_list:
@@ -142,41 +137,37 @@ class Connector:
             musicFolderId = musicFolderId))
 
     def buildSongUrl(self, song_id : str) -> str:
-        connection = self.__connect()
-        qdict = connection._getBaseQdict()
-        return "{}/rest/stream?u={}&s={}&t={}&c={}&v={}&id={}".format(
-            self.__createBaseUrlWithPort(),
-            self.__get_config(Connector.__KEY_USERNAME),
-            qdict["s"], # salt
-            qdict["t"], # token
-            self.__get_config(Connector.__KEY_APP_NAME),
-            self.__get_config(Connector.__KEY_API_VERSION),
-            song_id);
+        return self.__buildUrl(
+            verb = "stream",
+            url_dict = {"id" : song_id})
 
     def buildCoverArtUrl(self, item_id : str) -> str:
-        return self.__buildUrl(verb = "getCoverArt", item_id = item_id)
+        return self.__buildUrl(
+            verb = "getCoverArt", 
+            url_dict = {"id" : item_id})
 
-    def __buildUrl(self, verb, item_id : str) -> str:
+    def __addAuthParameters(self, url_dict : dict[str, str] = {}) -> dict[str, str]:
         connection = self.__connect()
         qdict = connection._getBaseQdict()
-        return "{}/rest/{}?u={}&s={}&t={}&c={}&v={}&id={}".format(
-            self.__createBaseUrlWithPort(),
-            verb,
-            self.__get_config(Connector.__KEY_USERNAME),
-            qdict["s"], # salt
-            qdict["t"], # token
-            self.__get_config(Connector.__KEY_APP_NAME),
-            self.__get_config(Connector.__KEY_API_VERSION),
-            item_id);
-            
-    def __get_config(self, name : str) -> str | None:
-        return (self.__config__[name] 
-            if name in self.__config__ 
-            else None)
+        url_dict["u"] = self.__configuration.getUserName()
+        url_dict["s"] = qdict["s"] #salt
+        url_dict["t"] = qdict["t"] #token
+        url_dict["c"] = self.__configuration.getAppName()
+        url_dict["v"] = self.__configuration.getApiVersion()
+        return url_dict
     
+    def __buildUrl(self, verb, url_dict : dict[str, str] = {}) -> str:
+        url = "{}/rest/{}".format(self.__createBaseUrlWithPort(), verb)
+        url_dict = self.__addAuthParameters(url_dict)
+        first : bool = True
+        for key in url_dict: 
+            url += "{}{}={}".format("?" if first else "&", key, url_dict[key])
+            first = False
+        return url
+            
     def __createBaseUrlWithPort(self):
-        baseUrl = self.__get_config(Connector.__KEY_BASE_URL)
-        port = self.__get_config(Connector.__KEY_PORT)
+        baseUrl = self.__configuration.getBaseUrl()
+        port = self.__configuration.getPort()
         url = baseUrl
         if ((baseUrl and baseUrl.startswith("https://") and port != 443) or 
             (baseUrl and baseUrl.startswith("http://") and port != 80)):
@@ -185,9 +176,9 @@ class Connector:
             
     def __connect(self):
         return libsonic.Connection(
-            baseUrl = self.__get_config(Connector.__KEY_BASE_URL), 
-            username = self.__get_config(Connector.__KEY_USERNAME), 
-            password = self.__get_config(Connector.__KEY_PASSWORD), 
-            port = int(str(self.__get_config(Connector.__KEY_PORT))),
-            appName = str(self.__get_config(Connector.__KEY_APP_NAME)),
-            apiVersion = str(self.__get_config(Connector.__KEY_API_VERSION)))
\ No newline at end of file
+            baseUrl = self.__configuration.getBaseUrl(), 
+            username = self.__configuration.getUserName(), 
+            password = self.__configuration.getPassword(), 
+            port = int(str(self.__configuration.getPort())),
+            appName = str(self.__configuration.getAppName()),
+            apiVersion = str(self.__configuration.getApiVersion()))
\ No newline at end of file
